import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  format,
  line,
  area,
  curveBasis,
  timeParse,
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// Formatting the total value to show on the Areachart
const xAxisTickFormat = (number) => format('.3s')(number);

const render = (data) => {
  const xValue = (d) => d.Year;
  const xAxisLabel = 'Year';

  const yValue = (d) => d.Vancouver;
  const yAxisLabel = 'Temperature (K)';

  const margin = {
    top: 10,
    right: 40,
    bottom: 108,
    left: 150,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // Determing the Scale for X axis
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  // Determing the Scale for Y axis
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();

  // Positioning the svg element
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // Assigning values to x axis
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);

  // Assigning values to y axis
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(xAxisTickFormat);

  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();

  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -93)
    .attr('x', -innerHeight / 2)
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);

  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 75)
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);
  // To draw the line on the curve   and adding it to svg
  const lineGenerator = line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(yValue(d)))
    .curve(curveBasis);

  g.append('path')
    .attr('class', 'line-chart')
    .attr('d', lineGenerator(data));

  // Creating a linear gradient for the area under the graph
  const defs = svg.append('defs');
  const gradient = defs
    .append('linearGradient')
    .attr('id', 'area-gradient')
    .attr('gradientTransform', 'rotate(90)');

  gradient
    .append('stop')
    .attr('offset', '0%')
    .attr('stop-color', 'purple');

  gradient
    .append('stop')
    .attr('offset', '100%')
    .attr('stop-color', 'transparent');

  // To draw the area under the curve and adding it to svg with gradient fill
  const areaGenerator = area()
    .x((d) => xScale(xValue(d)))
    .y0(innerHeight)
    .y1((d) => yScale(yValue(d)))
    .curve(curveBasis);

  g.append('path')
    .attr('class', 'area-chart')
    .attr('d', areaGenerator(data))
    .style('fill', 'url(#area-gradient)');
};

// Loading and Parsing CSV data
csv(
  'https://gist.githubusercontent.com/Yashwanth0402/e97c111c9f158eeb602089ce5500570a/raw/c4bcca46d449b75b0a9d644ae55497e3dd786bfc/gistfile1.txt',
).then((data) => {
  data.forEach((d) => {
    d.Vancouver = +d.Vancouver;
    d.Year = timeParse('%Y')(d.Year);
  });
  render(data);
});
